import numpy as np
import pylab
import matplotlib.pyplot as plt
from mpl_toolkits.mplot3d import Axes3D
import matplotlib.colors as mcolors
import matplotlib.cm as cm
from skimage.color import rgb2lab
from matplotlib.colors import LinearSegmentedColormap
from colorspacious import cspace_convert
from matplotlib.cm import get_cmap


def plot_colormap_gradient(colormap_name, save_path=None):
    if not isinstance(colormap_name, str):
        title = colormap_name.name
    else:
        title = colormap_name

    gradient = np.linspace(0, 1, 256)
    gradient = np.vstack((gradient, gradient))

    fig, ax = plt.subplots(figsize=(6, 2))
    ax.imshow(gradient, aspect='auto', cmap=plt.get_cmap(colormap_name))
    ax.set_axis_off()
    plt.title(f"Gradient of the '{title}' colormap")
    
    if save_path:
        plt.savefig(save_path)
    return ax
    


def plot_colormap_in_rgb_space(colormap_name, num_samples=500, save_path=None):
    if not isinstance(colormap_name, str):
        title = colormap_name.name
    else:
        title = colormap_name

    cmap = plt.get_cmap(colormap_name)
    colors = cmap(np.linspace(0, 1, num_samples))

    #get rgb values
    r = colors[:, 0]
    g = colors[:, 1]
    b = colors[:, 2]

    fig = plt.figure(figsize=(10, 8))
    ax = fig.add_subplot(111, projection='3d')

    for i in range(num_samples):
        ax.scatter(r[i], g[i], b[i], color=[r[i], g[i], b[i]], s=10)

    ax.set_xlabel('Red')
    ax.set_ylabel('Green')
    ax.set_zlabel('Blue')
    ax.set_title(f'Colormap "{title}" in RGB Space')
    ax.set_facecolor('white')
    ax.xaxis.pane.fill = False
    ax.yaxis.pane.fill = False
    ax.zaxis.pane.fill = False
    if save_path:
        plt.savefig(save_path)
        
    return ax


def create_custom_colormap(num_points=256, title="CustomColormap"):
    t = np.linspace(0, 1, num_points)

    #parametric functions for R, G, and B
    R = 0.3 * np.sin(2 * np.pi * t) + 0.3
    G = 0.3 * np.cos(2 * np.pi * t + np.pi / 3) + 0.2
    B = 0.2 * np.cos(2 * np.pi * t + 2 * np.pi / 3) + 0.6

    colors = np.stack([R, G, B], axis=1)

    #ensure colors are within valid RGB range (0-1)
    colors = np.clip(colors, 0, 1)

    return LinearSegmentedColormap.from_list(title, colors)



def delta_e(colormap_name, num_points=256, save_path=None):
    if isinstance(colormap_name, str):
        cmap = plt.get_cmap(colormap_name)
        title = colormap_name
    else:
        cmap = colormap_name
        title = colormap_name.name
    colors = cmap(np.linspace(0, 1, num_points))
    colors_rgb = colors[:, :3].reshape((num_points, 1, 3))
    colors_lab = rgb2lab(colors_rgb).reshape(num_points, 3)

    delta_es = []
    for i in range(len(colors_lab)-1):
        calc = abs(colors_lab[i][0] - colors_lab[i+1][0]) ** 2
        calc2 = abs(colors_lab[i][1] - colors_lab[i+1][1]) ** 2
        calc3 = abs(colors_lab[i][2] - colors_lab[i+1][2]) ** 2
        delta_es.append(np.sqrt(calc + calc2 + calc3))

    data_steps = np.linspace(0, 1, num_points)
    step_sizes = np.diff(data_steps)
    normalized_delta_es = delta_es / step_sizes

    #plotting
    # fig, ax = plt.figure(figsize=(10, 6))
    fig, ax = plt.subplots(figsize=(10, 6))
    x_values = (data_steps[:-1] + data_steps[1:]) / 2
    plt.plot(x_values, normalized_delta_es, marker='', linestyle='-')
    plt.plot([0,1], [200,200], marker='', linestyle='dashed', color='grey')
    plt.plot([0,1], [100,100], marker='', linestyle='dashed', color='grey')
    plt.xlabel('Data Value')
    plt.ylabel('ΔE (Perceptual Difference)')
    plt.title(f'ΔE Across "{title}" Colormap')
    plt.grid(True)
    
    if save_path:
        plt.savefig(save_path)
    return ax


def delta_e_lightness(colormap_name, num_points=256, save_path=None):
    if isinstance(colormap_name, str):
        cmap = plt.get_cmap(colormap_name)
        title = colormap_name
    else:
        cmap = colormap_name
        title = colormap_name.name

    colors = cmap(np.linspace(0, 1, num_points))
    colors_rgb = colors[:, :3].reshape((num_points, 1, 3))
    colors_lab = rgb2lab(colors_rgb).reshape(num_points, 3)

    delta_es = []
    for i in range(len(colors_lab)-1):
        calc = abs(colors_lab[i][0] - colors_lab[i+1][0]) ** 2
        delta_es.append(np.sqrt(calc))

    data_steps = np.linspace(0, 1, num_points)
    step_sizes = np.diff(data_steps)
    # print(step_sizes)
    normalized_delta_es = delta_es / step_sizes

    #plotting
    fig, ax = plt.subplots(figsize=(10, 6))
    x_values = (data_steps[:-1] + data_steps[1:]) / 2
    plt.plot(x_values, normalized_delta_es, marker='', linestyle='-')
    plt.plot([0,1], [200,200], marker='', linestyle='dashed', color='grey')
    plt.plot([0,1], [100,100], marker='', linestyle='dashed', color='grey')
    plt.ylim(0,600)
    plt.xlabel('Data Value')
    plt.ylabel('ΔE for Lightness')
    plt.title(f'ΔE Lightness "{title}" Colormap')
    plt.grid(True)
    if save_path:
        plt.savefig(save_path)
    return ax


def convert_colormap_for_colorblindness(colormap, cvd_type='deuteranomaly', num_points=256):
    # Get the colormap if provided by name
    if isinstance(colormap, str):
        cmap = plt.get_cmap(colormap)
        title = colormap
    else:
        cmap = colormap
        title = colormap.name
    
    original_colors = cmap(np.linspace(0, 1, num_points))
    original_colors_rgb = original_colors[:, :3]
    
    cvd_space = {"name": "sRGB1+CVD","cvd_type": cvd_type,"severity": 100}
    cvd_colors = cspace_convert(original_colors_rgb, cvd_space, "sRGB1")
    cvd_colors = np.clip(cvd_colors, 0, 1)
    
    new_cmap = LinearSegmentedColormap.from_list(f"{cvd_type}_{title}", cvd_colors, N=num_points)
    return new_cmap



def compare_colormaps(cmap1, cmap2, save_path=None):
    if isinstance(cmap1, str):
        title1 = cmap1
    else:
        title1 = cmap1.name
    if isinstance(cmap2, str):
        title2 = cmap2
    else:
        title2 = cmap2.name

    gradient = np.linspace(0, 1, 256)
    gradient = np.vstack((gradient, gradient))
    fig, axs = plt.subplots(1, 2, figsize=(12, 6), constrained_layout=True)
    
    axs[0].imshow(gradient, aspect='auto', cmap=plt.get_cmap(cmap1))
    axs[0].set_title(title1)
    axs[0].axis('off')
    axs[1].imshow(gradient, aspect='auto', cmap=plt.get_cmap(cmap2))
    axs[1].set_title(title2)
    axs[1].axis('off')
    
    if save_path:
        plt.savefig(save_path)
    return axs



if __name__ == "__main__":
    gradient = plot_colormap_gradient('viridis', save_path="viridis_gradient.png")
    plot_colormap_in_rgb_space('viridis', save_path="viridis_rgb.png")
    compare_colormaps('viridis', 'rainbow')
    
    # #delta e
    delta_e('rainbow', save_path="rainbow_delta_e.png")
    delta_e('viridis')
    lightness = delta_e_lightness('viridis', save_path="viridis_lightness.png")

    # #colorblindness
    deuter_colormap = convert_colormap_for_colorblindness('viridis', cvd_type='deuteranomaly')
    delta_e(deuter_colormap)
    compare_colormaps('viridis', deuter_colormap, save_path="viridis_vs_colorblind.png")
    
    # #custom c map
    custom_cmap = create_custom_colormap(title='Molly')
    plot_colormap_gradient(custom_cmap)
    plot_colormap_in_rgb_space(custom_cmap)
    deuter_colormap = convert_colormap_for_colorblindness(custom_cmap, cvd_type='deuteranomaly')
    ax = compare_colormaps(custom_cmap, deuter_colormap, save_path="comparison.png")
    delta_e(custom_cmap)
    delta_e_lightness(custom_cmap)


import numpy as np
import matplotlib.pyplot as plt
import pandas as pd
import landborn

def collides(x1,y1,x2,y2, r):
    return np.sqrt((x1-x2)**2 + (y1-y2)**2) < 2 * r

def check_collisions(candidate_ys, collisions, x, r):
    candidate_ys = sorted(candidate_ys, key=np.abs)
    for y in candidate_ys:
        has_collision = False
        for (x_prime, y_prime) in collisions:
            if collides(x, y, x_prime, y_prime, r): 
                has_collision = True
                break
        if not has_collision:
            return y

def swarmplot_inner(X, C, y_offset = 0, r=0.05, ax=None):
    Y = np.zeros_like(X)
    if ax == None:
        fig, ax = plt.subplots()
    for i in range(len(X)):
        collisions = []
        for j in range(i-1,0,-1):
            if X[i] - X[j] > 2 * r:
                break
            else:
                collisions.append((X[j], Y[j]))

        candidate_ys = [0]
        for (x_prime, y_prime) in collisions:
            candidate_ys.append(y_prime + np.sqrt(np.abs((2 * r) ** 2 - (X[i] - x_prime) ** 2)))
            candidate_ys.append(y_prime - np.sqrt(np.abs((2 * r) ** 2 - (X[i] - x_prime) ** 2)))
        
        Y[i] = check_collisions(candidate_ys, collisions, X[i], r)

    landborn.scatterplot(None, X, y_offset + Y, color=C, ax=ax)
    
def swarmplot(df, categorical_data, numerical_data, r=0.5, ax=None, save_path=None):
    if ax == None:
        fig, ax = plt.subplots()
    categories = df[categorical_data].unique()
    counter = 0
    colors = ['red', 'green', 'blue']
    for category in categories:
        passed_data = list(df.loc[df[categorical_data] == category][numerical_data])
        swarmplot_inner(passed_data, colors[counter], y_offset=counter,r=r, ax=ax)
        counter += 1
    ax.set_ylabel(categorical_data)
    ax.set_xlabel(numerical_data)
    plt.gca().set_aspect('equal', adjustable='box')
    if save_path:
        plt.savefig(save_path)

if __name__ == '__main__':
    np.random.seed(120)
    data = pd.DataFrame({
    'Category': ['A']*80 + ['B']*80 + ['C']*80,
    'Value': np.concatenate([np.random.randint(0, 20, size=80), np.random.randint(20, 30, size=80), np.random.randint(30, 50, size=80)])
    })
    swarmplot(data, 'Category', 'Value', r=0.8, save_path='test_swarmplot_confirmed.png')


from .functions import barplot, lineplot, scatterplot, jointplot, swarmplot
from .colormap import plot_colormap_gradient, plot_colormap_in_rgb_space, create_custom_colormap, delta_e, delta_e_lightness, convert_colormap_for_colorblindness, compare_colormaps

__version__ = '0.2.2'
__author__ = 'Molly Nelson'

print("Package Landborn 0.2.2 Successfully Imported")


import numpy as np
import matplotlib.pyplot as plt
import matplotlib.lines as mlines
import pandas as pd
import matplotlib.patches as patches
from matplotlib.path import Path

def barplot(df, xvar, yvar, orientation='vertical', color='lightblue', axis=None, save_path=None):
    if axis is None:
        fig, axis = plt.subplots()

    if orientation == 'vertical':
        if isinstance(xvar, str):
            categories = df[xvar].unique()
            x = np.arange(len(categories))
            heights = df.groupby(xvar)[yvar].mean()
            errors = df.groupby(xvar)[yvar].sem()
        else:
            x = xvar
            heights = x.mean()
            errors = x.sem()
        bars = []
        for i, (category, height) in enumerate(zip(categories, heights)):
            error_line = mlines.Line2D([x[i], x[i]], [height-(errors[category]/2), height+(errors[category]/2)], color='black')
            error_line_top = mlines.Line2D([x[i] - 0.1, x[i] + 0.1], [height+(errors[category]/2), height+(errors[category]/2)], color='black')
            error_line_bottom = mlines.Line2D([x[i] - 0.1, x[i] + 0.1], [height-(errors[category]/2), height-(errors[category]/2)], color='black')
            rect = patches.Rectangle((x[i] - 0.4, 0), 0.8, height,facecolor=color, edgecolor='black')
            bars.append(rect)
            axis.add_patch(rect)
            axis.add_line(error_line)
            axis.add_line(error_line_top)
            axis.add_line(error_line_bottom)

        axis.set_xticks(x)
        axis.set_xticklabels(categories)
        axis.set_xlabel(xvar)
        axis.set_ylabel(yvar)
        axis.autoscale()
        axis.set_ylim(bottom=0)

    elif orientation == 'horizontal':
        # Horizontal bar plot
        categories = df[yvar].unique()
        y = np.arange(len(categories))
        heights = df.groupby(yvar)[xvar].mean()
        errors = df.groupby(yvar)[xvar].sem()

        bars = []
        for i, (category, height) in enumerate(zip(categories, heights)):
            rect = patches.Rectangle((0, y[i] - 0.4), height, 0.8,facecolor=color, edgecolor='black')
            bars.append(rect)
            axis.add_patch(rect)

        axis.set_yticks(y)
        axis.set_yticklabels(categories)
        axis.set_ylabel(yvar)
        axis.set_xlabel(xvar)
        axis.autoscale()
        axis.set_xlim(0)
    if save_path:
        plt.savefig(save_path)

    return axis


def lineplot(df, x=[], y=[], color='k', size=1, style='-', marker=None, ax=None, save_path=None):
    if ax is None:
        fig, ax = plt.subplots()

    if isinstance(x, str) and df is not None:
        xdata = df[x]
    else:
        xdata = x
    if isinstance(y, str):
        ydata = df[y]
    else:
        ydata = y

    line = mlines.Line2D(xdata, ydata, color=color, linestyle=style, marker=marker, markersize=size)
    ax.add_line(line)
    ax.autoscale()
    
    if save_path:
        plt.savefig(save_path)

    return ax


def scatterplot(df, xvar, yvar, color='k',colormap='viridis', size=1, marker='.', ax=None, save_path=None):
    if ax is None:
        fig, ax = plt.subplots()
    if isinstance(xvar, str):
        xdata = df[xvar]
    else:
        xdata = xvar
        
    if isinstance(yvar, str):
        ydata = df[yvar]
    else:
        ydata = yvar
    
    patches_li = []
    for i, (x, y) in enumerate(zip(xdata, ydata)):
        #creating small square for "point"
        verts = [
        (x, y),  #left, bottom
        (x, y + .5),  #left, top
        (x + .5, y + .5),  #right, top
        (x + .5, y),  #right, bottom
        (x, y),  #back to start
        ]

        codes = [
            Path.MOVETO,
            Path.LINETO,
            Path.LINETO,
            Path.LINETO,
            Path.CLOSEPOLY,
        ]
        path = Path(verts, codes)
        patch = patches.PathPatch(path, lw=1, color=color)
        patches_li.append(patch)
    
    for patch in patches_li:
        ax.add_patch(patch)
    ax.set_xlim(min(xdata), max(xdata))
    ax.set_ylim(min(ydata), max(ydata))
    ax.autoscale()
    ax.legend()
    
    if save_path:
        plt.savefig(save_path)
        
    # plt.show()
    return ax


def jointplot(x, y, ax=None, color='black', title='Joint Plot', save_path=None):
    fig, axs = plt.subplots(2, 1, figsize=(10, 8), sharex=True) #create two plots

    #top line plot
    axs[0].set_title(title)
    axs[0].tick_params(axis='x',which='both', bottom=False,top=False,labelbottom=False)
    axs[0].spines['bottom'].set_visible(False)
    axs[0].spines['right'].set_visible(False)
    axs[0].spines['top'].set_visible(False)
    lineplot(None,x,y, color=color, ax=axs[0])

    #bottom scatterplot
    axs[1].spines['top'].set_visible(False)
    axs[1].spines['right'].set_visible(False)
    axs[1].set_xlabel('X axis')
    scatterplot(None, x,y, color=color, ax=axs[1])
    
    if save_path:
        plt.savefig(save_path)


def collides(x1,y1,x2,y2, r):
    return np.sqrt((x1-x2)**2 + (y1-y2)**2) < 2 * r

def check_collisions(candidate_ys, collisions, x, r):
    candidate_ys = sorted(candidate_ys, key=np.abs)
    for y in candidate_ys:
        has_collision = False
        for (x_prime, y_prime) in collisions:
            if collides(x, y, x_prime, y_prime, r): 
                has_collision = True
                break
        if not has_collision:
            return y

def swarmplot_inner(X, C, y_offset = 0, r=0.05, ax=None):
    Y = np.zeros_like(X)
    if ax == None:
        fig, ax = plt.subplots()
    for i in range(len(X)):
        collisions = []
        for j in range(i-1,0,-1):
            if X[i] - X[j] > 2 * r:
                break
            else:
                collisions.append((X[j], Y[j]))

        candidate_ys = [0]
        for (x_prime, y_prime) in collisions:
            candidate_ys.append(y_prime + np.sqrt(np.abs((2 * r) ** 2 - (X[i] - x_prime) ** 2)))
            candidate_ys.append(y_prime - np.sqrt(np.abs((2 * r) ** 2 - (X[i] - x_prime) ** 2)))
        
        Y[i] = check_collisions(candidate_ys, collisions, X[i], r)

    scatterplot(None, X, y_offset + Y, color=C, ax=ax)
    
def swarmplot(df, categorical_data, numerical_data, r=0.5, ax=None, save_path=None):
    if ax == None:
        fig, ax = plt.subplots()
    categories = df[categorical_data].unique()
    counter = 0
    colors = ['red', 'green', 'blue']
    for category in categories:
        passed_data = list(df.loc[df[categorical_data] == category][numerical_data])
        swarmplot_inner(passed_data, colors[counter], y_offset=counter,r=r, ax=ax)
        counter += 1
    ax.set_ylabel(categorical_data)
    ax.set_xlabel(numerical_data)
    plt.gca().set_aspect('equal', adjustable='box')
    if save_path:
        plt.savefig(save_path)

if __name__ == '__main__':
    np.random.seed(120)
    data = pd.DataFrame({
    'Category': ['A']*80 + ['B']*80 + ['C']*80,
    'Value': np.concatenate([np.random.randint(0, 20, size=80), np.random.randint(20, 30, size=80), np.random.randint(30, 50, size=80)])
    })
    swarmplot(data, 'Category', 'Value', r=0.8, save_path='test_swarmplot_confirmed.png')


import matplotlib.pyplot as plt
import matplotlib.lines as mlines
import matplotlib.patches as mpatches
from .scatterplot import scatterplot
from .lineplot import lineplot

def jointplot(x, y, ax=None, color='black', title='Joint Plot', save_path=None):
    fig, axs = plt.subplots(2, 1, figsize=(10, 8), sharex=True) #create two plots

    #top line plot
    axs[0].set_title(title)
    axs[0].tick_params(axis='x',which='both', bottom=False,top=False,labelbottom=False)
    axs[0].spines['bottom'].set_visible(False)
    axs[0].spines['right'].set_visible(False)
    axs[0].spines['top'].set_visible(False)
    lineplot(None,x,y, color=color, ax=axs[0])

    #bottom scatterplot
    axs[1].spines['top'].set_visible(False)
    axs[1].spines['right'].set_visible(False)
    axs[1].set_xlabel('X axis')
    scatterplot(None, x,y, color=color, ax=axs[1])
    
    if save_path:
        plt.savefig(save_path)



if __name__ == "__main__":
    x = [1, 14, 13, 14, 15]
    y = [10, 15, 1, 10, 15]
    
    ax = jointplot(x, y, color='blue',save_path="test_jointplot.png")


import matplotlib.pyplot as plt
import matplotlib.lines as mlines
import matplotlib.patches as mpatches


def lineplot(df, x=[], y=[], color='k', size=1, style='-', marker=None, ax=None, save_path=None):
    if ax is None:
        fig, ax = plt.subplots()

    if isinstance(x, str) and df is not None:
        xdata = df[x]
    else:
        xdata = x
    if isinstance(y, str):
        ydata = df[y]
    else:
        ydata = y

    line = mlines.Line2D(xdata, ydata, color=color, linestyle=style, marker=marker, markersize=size)
    ax.add_line(line)
    ax.autoscale()
    
    if save_path:
        plt.savefig(save_path)

    return ax

if __name__ == "__main__":
    x = [1, 2, 3, 4, 5]
    y = [10, 15, 7, 10, 5]
    plt = lineplot(None, x, y,save_path="test_lineplot.png")


import numpy as np
import matplotlib.pyplot as plt
import matplotlib.patches as mpatches
import matplotlib.lines as mlines
import pandas as pd

def barplot(df, xvar, yvar, orientation='vertical', color='lightblue', axis=None, save_path=None):
    if axis is None:
        fig, axis = plt.subplots()

    if orientation == 'vertical':
        if isinstance(xvar, str):
            categories = df[xvar].unique()
            x = np.arange(len(categories))
            heights = df.groupby(xvar)[yvar].mean()
            errors = df.groupby(xvar)[yvar].sem()
        else:
            x = xvar
            heights = x.mean()
            errors = x.sem()
        bars = []
        for i, (category, height) in enumerate(zip(categories, heights)):
            error_line = mlines.Line2D([x[i], x[i]], [height-(errors[category]/2), height+(errors[category]/2)], color='black')
            error_line_top = mlines.Line2D([x[i] - 0.1, x[i] + 0.1], [height+(errors[category]/2), height+(errors[category]/2)], color='black')
            error_line_bottom = mlines.Line2D([x[i] - 0.1, x[i] + 0.1], [height-(errors[category]/2), height-(errors[category]/2)], color='black')
            rect = mpatches.Rectangle((x[i] - 0.4, 0), 0.8, height,facecolor=color, edgecolor='black')
            bars.append(rect)
            axis.add_patch(rect)
            axis.add_line(error_line)
            axis.add_line(error_line_top)
            axis.add_line(error_line_bottom)

        axis.set_xticks(x)
        axis.set_xticklabels(categories)
        axis.set_xlabel(xvar)
        axis.set_ylabel(yvar)
        axis.autoscale()
        axis.set_ylim(bottom=0)

    elif orientation == 'horizontal':
        # Horizontal bar plot
        categories = df[yvar].unique()
        y = np.arange(len(categories))
        heights = df.groupby(yvar)[xvar].mean()
        errors = df.groupby(yvar)[xvar].sem()

        bars = []
        for i, (category, height) in enumerate(zip(categories, heights)):
            rect = mpatches.Rectangle((0, y[i] - 0.4), height, 0.8,facecolor=color, edgecolor='black')
            bars.append(rect)
            axis.add_patch(rect)

        axis.set_yticks(y)
        axis.set_yticklabels(categories)
        axis.set_ylabel(yvar)
        axis.set_xlabel(xvar)
        axis.autoscale()
        axis.set_xlim(0)
    if save_path:
        plt.savefig(save_path)

    return axis



if __name__ == "__main__":
    x = [1, 2, 3, 4, 5]
    y = [10, 15, 7, 10, 5]
    # plt = barplot(None, x, y, orientation="vertical" ,save_path="test_scatterplot.png")
    
    df = pd.DataFrame({
        'data values': np.random.normal(5,1,100),
        'categories': np.random.choice(['a','b','c'],replace=True, size=100)
    })
    df.loc[df['categories'] == 'a','data values'] = df.loc[df['categories'] == 'a']['data values'] * 2 - 6
    ax = barplot(df, 'categories', 'data values', orientation='vertical', save_path="test_barplot.png")
    
    


import matplotlib.patches as patches
from matplotlib.path import Path
import matplotlib.pyplot as plt

def scatterplot(df, xvar, yvar, color='k',colormap='viridis', size=1, marker='.', ax=None, save_path=None):
    if ax is None:
        fig, ax = plt.subplots()
    if isinstance(xvar, str):
        xdata = df[xvar]
    else:
        xdata = xvar
        
    if isinstance(yvar, str):
        ydata = df[yvar]
    else:
        ydata = yvar
    x_data_length = max(xdata) - min(xdata)
    x_patch_length = (x_data_length / len(xdata)) / 10
    print(x_patch_length)
    
    y_data_length = max(ydata) - min(ydata)
    y_patch_length = (y_data_length / len(ydata)) / 10
    print(y_patch_length)
    patches_li = []
    for i, (x, y) in enumerate(zip(xdata, ydata)):
        #creating small square for "point"
        verts = [
        (x - x_patch_length, y - y_patch_length),  #left, bottom
        (x - x_patch_length, y + y_patch_length),  #left, top
        (x + x_patch_length, y + y_patch_length),  #right, top
        (x + x_patch_length, y - y_patch_length),  #right, bottom
        (x, y),  #back to start
        ]

        codes = [
            Path.MOVETO,
            Path.LINETO,
            Path.LINETO,
            Path.LINETO,
            Path.CLOSEPOLY,
        ]
        path = Path(verts, codes)
        patch = patches.PathPatch(path, lw=1, color=color)
        patches_li.append(patch)
    
    for patch in patches_li:
        ax.add_patch(patch)
    ax.set_xlim(min(xdata), max(xdata))
    ax.set_ylim(min(ydata), max(ydata))
    # ax.autoscale()
    ax.legend()
    
    if save_path:
        plt.savefig(save_path)
        
    # plt.show()
    return ax

if __name__ == "__main__":
    x = [1, 2, 3, 4, 5]
    y = [10, 15, 7, 10, 5]
    plt = scatterplot(None, x, y,save_path="test_scatterplot.png")


from setuptools import find_packages, setup

setup(
    name='landborn',
    packages=find_packages(include=['landborn']),
    version='0.2.2',
    description='Landborn Visualization Library',
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    author='Molly Nelson',
    author_email="mollyynelson@gmail.com",
    license='MIT',
    install_requires=['pandas', 'numpy', 'matplotlib'],
    setup_requires=['pytest-runner'],
    tests_require=['pytest==8.0.0'],
    test_suite='tests'
)